package com.bipt.intelligentapplicationorchestrationservice.service;

import com.bipt.intelligentapplicationorchestrationservice.entity.DeployRequest;
import com.bipt.intelligentapplicationorchestrationservice.entity.DeployResponse;
import com.bipt.intelligentapplicationorchestrationservice.entity.ModelInfo;
import com.bipt.intelligentapplicationorchestrationservice.entity.GpuResource;
import com.bipt.intelligentapplicationorchestrationservice.exception.DeployException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


@Service
public class ModelDeployer {

    @Autowired
    private ModelRepositoryClient modelRepositoryClient;

    @Autowired
    private ModelEvaluator modelEvaluationService;

    @Autowired
    private NormalDeployService normalDeployService;

    @Autowired
    private GrayDeployService grayDeployService;

    @Autowired
    private ResourceAllocator resourceAllocator;

    @Autowired
    private GpuResourceService gpuResourceService;

    public DeployResponse deploy(DeployRequest request) {
        try {
            // 1. 获取完整模型信息
            ModelInfo modelInfo = getModelInfo(request.getModelId());

            // 2. 更新请求中的模型信息
            request.setModelInfo(modelInfo);

            // 3. 添加模型评估检查
            if (!isEvaluationPassed(modelInfo)) {
                return DeployResponse.fail(403,
                        "模型[" + modelInfo.getModelName() + "]评估未通过，无法部署");
            }
            List<GpuResource> availableResources = gpuResourceService.getAvailableResources();
            int requiredMemory = request.getRequiredMemory();
            String modelId = request.getModelId();

            if (!resourceAllocator.checkResourceQuota(availableResources, requiredMemory, modelId)){
                return DeployResponse.fail(507, "资源配额不足。需要内存: " + requiredMemory + "MB\n");
            }

            // 4. 根据部署类型路由
            if (request.getDeployType() == DeployRequest.DeployType.NORMAL) {
                return normalDeployService.process(request);
            } else {
                return grayDeployService.process(request);
            }
        } catch (DeployException e) {
            return DeployResponse.fail(404, "获取模型信息失败: " + e.getMessage());
        } catch (Exception e) {
            return DeployResponse.fail(500, "部署失败: " + e.getMessage());
        }
    }

    /**
     * 从模型仓库获取模型详细信息
     */
    private ModelInfo getModelInfo(String modelId) {
        // 添加缓存和重试机制
        ModelInfo modelInfo = modelRepositoryClient.getModelInfo(modelId);

        // 验证关键字段
        if (modelInfo.getModelFilePath() == null) {
            throw new DeployException("模型文件路径未定义");
        }

        // 获取额外信息
        modelInfo.setStorageLocation(
                modelRepositoryClient.getModelStorageLocation(modelId)
        );

        return modelInfo;
    }

    /**
     * 检查模型评估状态
     */
    private boolean isEvaluationPassed(ModelInfo modelInfo) {
        return modelEvaluationService.isEvaluationPassed(
                modelInfo.getModelId()
        );
    }
}
