package com.bipt.intelligentapplicationorchestrationservice.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

@Component
public class ModelEvaluator {
    @Value("${model.evaluation.service-url}")
    private String evaluationServiceUrl;

    @Value("${model.evaluation.api-timeout:3000}")
    private int apiTimeout;

    @Autowired
    private RestTemplate restTemplate;

    /**
     * 检查模型评估是否通过
     * @param modelId 模型ID
     * @return 评估是否通过
     */
    public boolean isEvaluationPassed(String modelId) {
        return isEvaluationPassed(modelId, null);
    }

    /**
     * 检查模型评估是否通过（带版本号）
     * @param modelId 模型ID
     * @param version 模型版本
     * @return 评估是否通过
     */
    public boolean isEvaluationPassed(String modelId, String version) {
        // 1. 构建API调用参数
        Map<String, String> params = new HashMap<>();
        params.put("modelId", modelId);
        if (version != null) {
            params.put("version", version);
        }

        try {
            // 2. 设置请求头和超时
            HttpHeaders headers = new HttpHeaders();
            headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));
            headers.setContentType(MediaType.APPLICATION_JSON);
            headers.set("X-Request-ID", generateRequestId(modelId));

            // 3. 调用评估API
            ResponseEntity<EvaluationResponse> response = restTemplate.exchange(
                    buildEvaluationUrl(modelId, version),
                    HttpMethod.GET,
                    new HttpEntity<>(headers),
                    EvaluationResponse.class
            );

            // 4. 处理响应
            if (response.getStatusCode() == HttpStatus.OK && response.getBody() != null) {
                return response.getBody().isPassed();
            } else if (response.getStatusCode() == HttpStatus.NOT_FOUND) {
                logEvaluationWarning(modelId, "模型未找到");
            } else {
                logEvaluationError(modelId, "评估服务返回非200状态: " + response.getStatusCode());
            }
        } catch (Exception e) {
            logEvaluationError(modelId, "评估服务调用失败: " + e.getMessage());
        }

        // 5. 默认返回失败
        return false;
    }

    // ================= 辅助方法 =================

    /**
     * 构建评估API URL
     */
    private String buildEvaluationUrl(String modelId, String version) {
        String baseUrl = evaluationServiceUrl + "/models/" + modelId + "/evaluation";
        return version != null ? baseUrl + "?version=" + version : baseUrl;
    }

    /**
     * 生成请求ID（用于日志追踪）
     */
    private String generateRequestId(String modelId) {
        return "eval-" + modelId + "-" + System.currentTimeMillis();
    }

    /**
     * 记录评估错误日志
     */
    private void logEvaluationError(String modelId, String message) {
        // 实际实现应使用日志框架（如SLF4J）
        System.err.println("[" + modelId + "] 评估错误: " + message);
    }

    /**
     * 记录评估警告日志
     */
    private void logEvaluationWarning(String modelId, String message) {
        // 实际实现应使用日志框架
        System.out.println("[" + modelId + "] 评估警告: " + message);
    }

    /**
     * 评估响应内部类
     */
    private static class EvaluationResponse {
        private boolean passed;
        private String reason;
        private String evaluatedAt;

        // 需要默认构造方法用于JSON反序列化
        public EvaluationResponse() {}

        public boolean isPassed() {
            return passed;
        }

        public String getReason() {
            return reason;
        }

        public String getEvaluatedAt() {
            return evaluatedAt;
        }
    }
}
