package com.bipt.intelligentapplicationorchestrationservice.service;

import com.bipt.intelligentapplicationorchestrationservice.entity.ModelInfo;
import com.bipt.intelligentapplicationorchestrationservice.exception.DeployException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;


@Component
public class ModelRepositoryClient {

    @Value("${model.repository.url}")
    private String repositoryBaseUrl;

    @Value("${model.repository.api-key}")
    private String apiKey;

    @Autowired
    private RestTemplate restTemplate;

    /**
     * 从模型仓库获取模型信息
     *
     * @param modelId 模型唯一标识
     * @return 完整的模型信息对象
     */
    public ModelInfo getModelInfo(String modelId) {
        // 1. 构建请求URL
        String url = repositoryBaseUrl + "/models/" + modelId + "/info";

        // 2. 设置认证头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("X-API-KEY", apiKey);
        headers.set("Model-Id", modelId);

        HttpEntity<?> entity = new HttpEntity<>(headers);

        try {
            // 3. 发送请求到模型仓库服务
            ResponseEntity<ModelInfo> response = restTemplate.exchange(
                    url,
                    HttpMethod.GET,
                    entity,
                    ModelInfo.class
            );

            // 4. 处理响应
            if (response.getStatusCode() == HttpStatus.OK && response.getBody() != null) {
                return response.getBody();
            } else if (response.getStatusCode() == HttpStatus.NOT_FOUND) {
                throw new DeployException("模型未找到: " + modelId);
            } else {
                throw new DeployException("获取模型信息失败，状态码: " + response.getStatusCode());
            }
        } catch (Exception e) {
            throw new DeployException("访问模型仓库失败: " + e.getMessage(), e);
        }
    }
    public String getModelStorageLocation(String modelId) {
        // 1. 构建请求URL
        String url = repositoryBaseUrl + "/models/" + modelId + "/storage-location";

        // 2. 设置认证头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("X-API-KEY", apiKey);
        headers.setAccept(Collections.singletonList(MediaType.TEXT_PLAIN));

        HttpEntity<?> entity = new HttpEntity<>(headers);

        try {
            // 3. 发送请求到模型仓库服务
            ResponseEntity<String> response = restTemplate.exchange(
                    url,
                    HttpMethod.GET,
                    entity,
                    String.class
            );

            // 4. 处理响应
            if (response.getStatusCode() == HttpStatus.OK && response.getBody() != null) {
                return response.getBody();
            } else if (response.getStatusCode() == HttpStatus.NOT_FOUND) {
                throw new DeployException("模型存储位置未找到: " + modelId);
            } else {
                throw new DeployException("获取存储位置失败，状态码: " + response.getStatusCode());
            }
        } catch (Exception e) {
            throw new DeployException("访问模型仓库失败: " + e.getMessage(), e);
        }
    }
}