package com.bipt.intelligentapplicationorchestrationservice.service;

import com.bipt.intelligentapplicationorchestrationservice.entity.DeployRequest;
import com.bipt.intelligentapplicationorchestrationservice.entity.DeployResponse;
import com.bipt.intelligentapplicationorchestrationservice.entity.DeploymentResource;
import com.bipt.intelligentapplicationorchestrationservice.entity.ModelInfo;
import com.bipt.intelligentapplicationorchestrationservice.exception.EvaluationFailedException;
import com.bipt.intelligentapplicationorchestrationservice.util.TemplateParser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.PosixFilePermission;
import java.util.Set;
import java.util.concurrent.TimeUnit;
@Service
public class NormalDeployService {
        @Autowired
        private ModelEvaluator modelEvaluator;

        @Autowired
        private ResourceAllocator resourceAllocator;

        @Autowired
        private GpuResourceService gpuService;

        @Autowired
        private TemplateParser templateParser;  // 使用模板解析器

        @Value("${model.deploy.script-directory:/opt/deploy_scripts}")
        private String scriptDirectory;

        @Value("${model.deploy.script-timeout:300}")
        private int scriptTimeout;

        public DeployResponse process(DeployRequest request) {
            try {
                // 1. 检查模型评估状态（使用ModelInfo包含的模型ID）
                ModelInfo modelInfo = request.getModelInfo();
                if (!modelEvaluator.isEvaluationPassed(modelInfo != null ? modelInfo.getModelName() : "")) {
                    throw new EvaluationFailedException("模型评估未通过");
                }

                // 2. 分配资源并获取URL
                DeploymentResource resource = resourceAllocator.allocate(
                        gpuService.getAvailableResources(),
                        request.getRequiredMemory(),
                        modelInfo != null ? modelInfo.getModelName() : "unknown",
                        false
                );

                // 3. 使用ModelInfo生成部署脚本内容
                String scriptContent = templateParser.generateDeploymentScript(
                        request.getModelInfo(),
                        resource.getUrl()
                );

                // 4. 执行部署脚本
                executeDeploymentScript(scriptContent);

                // 5. 标记资源已分配
                gpuService.markAsAllocated(
                        resource.getGpu().getGPUId(),
                        request.getRequiredMemory()
                );

                return DeployResponse.success(resource.getUrl());

            } catch (EvaluationFailedException e) {
                return DeployResponse.fail(403, e.getMessage());
            } catch (Exception e) {
                return DeployResponse.fail(500, "部署失败: " + e.getMessage());
            }
        }

        /**
         * 执行部署脚本
         */
        private void executeDeploymentScript(String scriptContent) throws Exception {
            // 创建脚本文件
            Path scriptPath = Paths.get(scriptDirectory, "deploy_" + System.currentTimeMillis() + ".sh");
            Files.write(scriptPath, scriptContent.getBytes());

            // 设置执行权限 (754)
            Set<PosixFilePermission> permissions = Set.of(
                    PosixFilePermission.OWNER_READ,
                    PosixFilePermission.OWNER_WRITE,
                    PosixFilePermission.OWNER_EXECUTE,
                    PosixFilePermission.GROUP_READ,
                    PosixFilePermission.GROUP_EXECUTE,
                    PosixFilePermission.OTHERS_READ
            );
            Files.setPosixFilePermissions(scriptPath, permissions);

            // 执行脚本
            Process process = new ProcessBuilder(scriptPath.toString())
                    .directory(Paths.get(scriptDirectory).toFile())
                    .start();

            // 带超时等待
            boolean completed = process.waitFor(scriptTimeout, TimeUnit.SECONDS);
            if (!completed) {
                process.destroyForcibly();
                throw new RuntimeException("脚本执行超时（限制: " + scriptTimeout + "秒）");
            }

            // 检查退出码
            if (process.exitValue() != 0) {
                throw new RuntimeException("脚本执行失败，退出码: " + process.exitValue());
            }

            // 清理脚本文件
            Files.deleteIfExists(scriptPath);
        }
}
