package com.bipt.intelligentapplicationorchestrationservice.service;

import com.bipt.intelligentapplicationorchestrationservice.entity.DeploymentResource;
import com.bipt.intelligentapplicationorchestrationservice.entity.GpuResource;
import com.bipt.intelligentapplicationorchestrationservice.constant.ConfigConstants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.client.ResourceAccessException;

import java.util.Comparator;
import java.util.List;

@Component
public class ResourceAllocator {

    @Autowired
    private ConfigConstants config;

    // 检查资源配额
    public boolean checkResourceQuota(List<GpuResource> resources, int requiredMemory, String modelId) {
        // 1. 检查全局资源是否充足
        long totalFreeMemory = resources.stream()
                .mapToInt(this::getRemainingMemory)
                .sum();

        // 全局内存不足
        if (totalFreeMemory < requiredMemory) {
            return false;
        }

        // 2. 检查单个GPU是否能满足需求
        boolean canAllocate = resources.stream()
                .anyMatch(gpu -> getRemainingMemory(gpu) >= requiredMemory);

        if (!canAllocate) {
            // 检查碎片化分配
            return checkFragmentationAllocation(resources, requiredMemory);
        }

        return true;
    }

    // 检查碎片化分配可能性
    private boolean checkFragmentationAllocation(List<GpuResource> resources, int requiredMemory) {
        // 按碎片率排序（碎片小的优先）
        resources.sort(Comparator.comparingDouble(
                r -> (double)getRemainingMemory(r) / r.getGPUMaxMemory()
        ));

        // 检查碎片整合后是否可能满足需求
        int accumulatedMemory = 0;
        for (GpuResource gpu : resources) {
            int fragment = getRemainingMemory(gpu);
            // 如果单个碎片就足够，直接返回成功
            if (fragment >= requiredMemory) return true;

            // 累积碎片
            accumulatedMemory += fragment;
            if (accumulatedMemory >= requiredMemory) {
                return true;
            }
        }

        return false;
    }

    //获取剩余内存
    private int getRemainingMemory(GpuResource resource){
        return resource.getGPUMaxMemory()-resource.getGPUMemorySize();
    }

    public DeploymentResource allocate(
            List<GpuResource> resources,
            int requiredMemory,
            String modelId,
            boolean isGray
    ){
        resources.sort(Comparator.comparingInt(GpuResource::getGPUMemorySize));

        //第一轮分配
        for(GpuResource resource:resources){
            if(getRemainingMemory(resource) >= requiredMemory) {
                return createResource(resource, modelId, isGray);
            }
        }

        //第二轮分配
        return defragmentation(resources,requiredMemory, modelId, isGray);
    }


    private DeploymentResource defragmentation(
            List<GpuResource> resources,
            int requiredMemory,
            String modelId,
            boolean isGray
    ){
        //按内存碎片大小排序（最小碎片优先）
        resources.sort(Comparator.comparingDouble(
                r -> (double)getRemainingMemory(r) / r.getGPUMaxMemory()));

        for(GpuResource resource:resources){
            if(getRemainingMemory(resource) >= requiredMemory){
                return createResource(resource, modelId, isGray);
            }
        }
        throw new ResourceAccessException("GPU资源不足");
    }

    private DeploymentResource createResource(GpuResource gpu, String modelId, boolean isGray){
        String urlType = isGray ? "gray":"prod";
        String url = String.format(
                config.URL_TEMPLATE,
                gpu.getIp(),
                config.MODEL_PORT,
                modelId,
                urlType
        );
        return new DeploymentResource(gpu, url);
    }
}
