package com.bipt.intelligentapplicationorchestrationservice.service.impl;

import com.bipt.intelligentapplicationorchestrationservice.mapper.GpuResourceDao;
import com.bipt.intelligentapplicationorchestrationservice.mapper.GpuMapper;
import com.bipt.intelligentapplicationorchestrationservice.pojo.GpuCreateDTO;
import com.bipt.intelligentapplicationorchestrationservice.pojo.GpuResponseDTO;
import com.bipt.intelligentapplicationorchestrationservice.pojo.GpuUpdateDTO;
import com.bipt.intelligentapplicationorchestrationservice.entity.GpuResource;
import com.bipt.intelligentapplicationorchestrationservice.enumeration.ErrorCodeEnum;
import com.bipt.intelligentapplicationorchestrationservice.pojo.ResponseVO;
import com.bipt.intelligentapplicationorchestrationservice.service.GpuManageService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class GpuManageServiceImpl implements GpuManageService {
    private static final Logger logger = LoggerFactory.getLogger(GpuManageServiceImpl.class);

    @Autowired
    private GpuResourceDao gpuDao;

    @Autowired
    private GpuMapper gpuMapper;

    @Transactional
    @Override
    public ResponseVO createGpuResource(GpuCreateDTO dto) {
        try {
            logger.info("创建GPU资源请求: {}", dto);

            GpuResource entity = gpuMapper.toEntity(dto);
            gpuDao.insert(entity);

            logger.info("创建GPU资源成功, ID: {}", entity.getGPUId());
            return ResponseVO.success(entity);

        } catch (Exception e) {
            logger.error("创建GPU资源失败: {}, 参数: {}", e.getMessage(), dto, e);
            return ResponseVO.error(ErrorCodeEnum.INTERNAL_SERVER_ERROR);
        }
    }

    @Transactional
    @Override
    public ResponseVO deleteGpuResource(Long gpuId) {
        logger.info("删除GPU资源请求, ID: {}", gpuId);

        try {
            GpuResource entity = gpuDao.selectById(gpuId);
            if (entity == null) {
                logger.warn("GPU资源不存在, ID: {}", gpuId);
                return ResponseVO.error(ErrorCodeEnum.GPU_NOT_FOUND);
            }

            gpuDao.isDeleted(gpuId);
            logger.info("逻辑删除GPU资源成功, ID: {}", gpuId);
            return ResponseVO.success();

        } catch (Exception e) {
            logger.error("删除GPU资源失败, ID: {}, 错误: {}", gpuId, e.getMessage(), e);
            return ResponseVO.error(ErrorCodeEnum.INTERNAL_SERVER_ERROR);
        }
    }

    @Transactional
    @Override
    public void updateGpuResource(GpuUpdateDTO dto) {
        logger.info("更新GPU资源请求: {}", dto);

        try {
            GpuResource entity = gpuMapper.toEntity(dto);
            gpuDao.updateById(entity);
            logger.info("更新GPU资源成功, ID: {}", entity.getGPUId());

        } catch (Exception e) {
            logger.error("更新GPU资源失败: {}, 参数: {}", e.getMessage(), dto, e);
            throw new RuntimeException("更新GPU资源失败", e);
        }
    }

    @Override
    public List<GpuResponseDTO> searchByCriteria(String model, Integer memorySize, String ip) {
        Map<String, Object> params = new HashMap<>();
        if(model != null) params.put("model", "%" + model + "%");
        if(memorySize != null) params.put("memorySize", memorySize);
        if(ip != null) params.put("ip", ip);

        logger.info("查询GPU资源条件: {}", params);

        try {
            List<GpuResource> entities = gpuDao.selectByFields(params);
            logger.info("查询到 {} 条GPU资源", entities.size());

            return entities.stream()
                    .map(gpuMapper::toDTO)
                    .collect(Collectors.toList());

        } catch (Exception e) {
            logger.error("查询GPU资源失败, 条件: {}, 错误: {}", params, e.getMessage(), e);
            throw new RuntimeException("查询GPU资源失败", e);
        }
    }

    // ========== 新增的日志追踪方法 ==========

    private void logOperation(String methodName, Object... details) {
        if (logger.isDebugEnabled()) {
            StringBuilder logBuilder = new StringBuilder(methodName);
            for (Object detail : details) {
                logBuilder.append(" | ").append(detail);
            }
            logger.debug(logBuilder.toString());
        }
    }

    private void logDuration(String methodName, long startTime, boolean success) {
        long duration = System.currentTimeMillis() - startTime;
        String status = success ? "成功" : "失败";
        logger.info("方法 {} 执行{} | 耗时: {}ms", methodName, status, duration);
    }

    // ========== 增强的日志版本 ==========

    @Transactional
    @Override
    public ResponseVO createGpuResourceWithTrace(GpuCreateDTO dto) {
        long start = System.currentTimeMillis();
        try {
            logOperation("createGpuResource", "请求参数", dto);

            ResponseVO result = createGpuResource(dto);

            logDuration("createGpuResource", start, true);
            return result;

        } catch (Exception e) {
            logDuration("createGpuResource", start, false);
            throw e;
        }
    }
}
